package com.gic.enterprise.web.controller;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;

import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.open.api.service.AppTokenApiService;
import com.gic.open.api.service.market.MarketUserApiService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.OperationResultUtils;
import com.gic.enterprise.constant.billing.AutoRechargeRedisConstant;
import com.gic.enterprise.constant.billing.FeeTypeEnum;
import com.gic.enterprise.dto.BillingEnterpriseSmsPackageDTO;
import com.gic.enterprise.dto.SmsPackageDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.HomeStatisticsQO;
import com.gic.enterprise.service.*;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.vo.billing.OverviewPackageVO;

/**
 * 计费中心余额账户
 * @ClassName: BillingAccountController 
 * @Description:  
 * @author guojuxing 
 * @date 2019/8/16 3:53 PM 
 */
@RestController
@RequestMapping("/billing-account")
public class BillingAccountController {
    private static final Logger LOGGER = LoggerFactory.getLogger(BillingAccountController.class);
    @Autowired
    private BillingAccountApiService billingAccountApiService;
    @Autowired
    private PackageApiService packageApiService;
    @Autowired
    private HomeStatisticsApiService homeStatisticsApiService;
    @Autowired
    private BillingCouponCardApiService billingCouponCardApiService;
    @Autowired
    private BillingEnterpriseSmsPackageApiService billingEnterpriseSmsPackageApiService;
    @Autowired
    private MarketUserApiService marketUserApiService;
    @Autowired
    private AppTokenApiService appTokenApiService;

    /**
     * 账户余额相关信息
     * @Title: billingAccountInfo 
     * @Description: 
      * @author guojuxing  
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/billing-account-info")
    public RestResponse billingAccountInfo() {
        Integer enterprise = UserDetailUtils.getUserDetail().getEnterpriseId();
        return ResultControllerUtils.commonResult(billingAccountApiService.getByEnterpriseId(enterprise));
    }

    @RequestMapping("/list-package-type")
    public RestResponse listPackageType() {
        return ResultControllerUtils.commonResult(packageApiService.listAllSmsPackage());
    }

    @RequestMapping("/is-buy-sms")
    public RestResponse isBuySms() {
        Integer enterprise = UserDetailUtils.getUserDetail().getEnterpriseId();
        return ResultControllerUtils.commonResult(appTokenApiService.isBuySms(Long.parseLong(enterprise.toString())));
    }

    /**
     * 套餐余额少于XX的时候自动续充的下拉选择列表
     * @Title: listPackageTypeLimit 
     * @Description:
      * @author guojuxing  
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/list-package-type-limit")
    public RestResponse listPackageTypeLimit() {
        return ResultControllerUtils.commonResult(packageApiService.listAutoRechargeSmsPackage());
    }

    /**
     * 设置自动续订短信套餐包配置
     * @Title: setAutoRecharge 
     * @Description: 
      * @author guojuxing 
     * @param smsCode
     * @param preNumber 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/set-auto-recharge")
    public RestResponse setAutoRecharge(@NotBlank(message = "指定套餐包不能为空") String smsCode,
            @NotNull(message = "自动续订条件不能为空") Integer preNumber) {
        return OperationResultUtils.operationResult(
                billingAccountApiService.setAutoRecharge(smsCode, preNumber,
                        UserDetailUtils.getUserDetail().getEnterpriseId()),
                "自动续订短信套餐包配置", UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseName());
    }

    /**
     * 关闭自动续订短信套餐包配置
     * @Title: closeAutoRechargeConfig 
     * @Description:
      * @author guojuxing  
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/close-auto-recharge-config")
    public RestResponse closeAutoRechargeConfig() {
        return OperationResultUtils.operationResult(
                billingAccountApiService.closeAutoRechargeConfig(UserDetailUtils.getUserDetail().getEnterpriseId()),
                "关闭自动续订短信套餐包配置", UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseName());
    }

    /**
     * 资源监控相关信息-短信套餐包
     * @Title: countPackageInfo 
     * @Description:
      * @author guojuxing  
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/package-info")
    public RestResponse countPackageInfo() {
        Map<String, Object> resultMap = new HashMap<>(8);
        OverviewPackageVO vo = new OverviewPackageVO();

        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        //统计优惠券数量
        Integer couponCount = billingCouponCardApiService.countCoupon(enterpriseId).getResult();
        if (couponCount == 0) {
            vo.setCoupon("暂无优惠券");
        } else {
            vo.setCoupon("您有" + couponCount + "张优惠券待使用");
        }

        //我的套餐包统计数据
        List<BillingEnterpriseSmsPackageDTO> smsPackageDTOList = billingEnterpriseSmsPackageApiService
                .listSmsPackage(enterpriseId).getResult();
        //短信余额
        Integer smsBalance = 0;
        //已使用的短信数量
        Integer usedSms = 0;
        //可使用套餐包
        Integer smsPackageBalance = 0;
        //已使用的套餐包
        Integer usedSmsPackage = 0;
        //当前使用中的有效期
        String lastValidDate = null;
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        String today = format.format(new Date());
        for (BillingEnterpriseSmsPackageDTO smsPackageDTO : smsPackageDTOList) {
            String validDate = format.format(smsPackageDTO.getEffectiveDate());
            //是否过期
            boolean isExpired = validDate.compareTo(today) < 0;
            if (smsPackageDTO.getStatus() == 2) {
                //使用中
                lastValidDate = validDate;
            }
            //已使用的套餐包
            if (!isExpired) {
                if (smsPackageDTO.getRemainAccount() == 0) {
                    usedSmsPackage++;
                } else {
                    //可使用的
                    smsPackageBalance++;
                }
            }
            if (!isExpired) {
                usedSms += (smsPackageDTO.getTotalAcount() - smsPackageDTO.getRemainAccount());
                smsBalance += smsPackageDTO.getRemainAccount();
            }
        }

        vo.setHasBuyShortMessagePackage(smsPackageDTOList.size() > 0);
        vo.setPackageCount(smsPackageBalance);
        vo.setShortMessageBalance(smsBalance);
        vo.setPackageTotalCount(smsPackageBalance + usedSmsPackage);
        vo.setShortMessageTotalBalance(usedSms + smsBalance);
        vo.setLastValidDate(lastValidDate);

        //自动续订短信套餐包的配置信息
        ServiceResponse<Map<String, Object>> response = billingAccountApiService
                .getAutoRechargeConfig(UserDetailUtils.getUserDetail().getEnterpriseId());
        if (response.isSuccess()) {
            Map<String, Object> map = response.getResult();
            SmsPackageDTO smsPackageDTO = (SmsPackageDTO) map.get(AutoRechargeRedisConstant.SMS_PACKAGE_MAP_KEY);
            vo.setAutoRechargeDescription("套餐余额总数少于" + map.get("1") + "条时自动续充" + smsPackageDTO.getItemCount() + "条");

            resultMap.put("preNumber", map.get(AutoRechargeRedisConstant.PRE_NUMBER_MAP_KEY));
            resultMap.put("smsPackageInfo", smsPackageDTO);
        } else {
            vo.setAutoRechargeDescription("暂未开启自动续充");
            resultMap.put("preNumber", null);
            resultMap.put("smsPackageInfo", null);
        }

        resultMap.put("smsInfo", vo);
        return RestResponse.success(resultMap);
    }

    /**
     * 消费资源监控趋势图表
     * @Title: trendAnalysisChart 
     * @Description: 
      * @author guojuxing 
     * @param params 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/trend-analysis-chart")
    public RestResponse trendAnalysisChart(HomeStatisticsQO params) {
        params.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(homeStatisticsApiService.viewFeeCharts(params));
    }

    /**
     * 消费资源监控占比图表
     * @Title: feeProportion 
     * @Description: 
      * @author guojuxing 
     * @param dateType 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/fee-proportion")
    public RestResponse feeProportion(Integer dateType) {
        boolean isNotRightParamValue = dateType != 1 && dateType != 2;
        if (dateType == null || isNotRightParamValue) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误");
        }
        return ResultControllerUtils.commonResult(
                homeStatisticsApiService.getFee(UserDetailUtils.getUserDetail().getEnterpriseId(), dateType));
    }

    /**
     * 消费类型
     * @Title: listFeeType 
     * @Description: 
      * @author guojuxing  
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/list-fee-type")
    public RestResponse listFeeType() {
        Map<Integer, String> resultMap = new HashMap<>(16);
        for (FeeTypeEnum payTypeEnum : FeeTypeEnum.values()) {
            resultMap.put(payTypeEnum.getCode(), payTypeEnum.getMessage());
        }
        return RestResponse.success(resultMap);
    }

    /**
     * 单点登录到服务市场首页
     * @Title: ssoToMarketService 
     * @Description:
      * @author guojuxing
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("sso-to-market-service")
    public RestResponse ssoToMarketService() throws IOException {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<String> result = marketUserApiService
                .BindGicEnterprise(Long.valueOf(enterpriseId), null, null, null);
        if (result.isSuccess()) {
            return RestResponse.success("https://four.gicdev.com/market/gic" + "?loginKey=" + result.getResult()
                    + "&enterpriseId=" + enterpriseId);
        }
        return EnterpriseRestResponse.failure(result);
    }
}
