package com.gic.operation.web.controller;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.gic.auth.constant.UserConstants;
import com.gic.authcenter.security.core.dto.EnterpriseTempDTO;
import com.gic.authcenter.security.core.dto.UserDetailsVO;
import com.gic.enterprise.dto.EnterpriseResourceRelDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.EnterpriseQO;
import com.gic.enterprise.utils.AutoCreatePasswordUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.finance.constant.EnterprisePlatformTypeEnum;
import com.gic.finance.constant.OrderTypeEnum;
import com.gic.marketing.process.api.service.sms.SmsSendApiService;
import com.gic.open.api.dto.market.MarketUserDTO;
import com.gic.open.api.service.market.MarketAppCategoryApiService;
import com.gic.open.api.service.market.MarketUserApiService;
import com.gic.operation.web.vo.ApplicationTokenVO;
import com.gic.operation.web.vo.CouponLogVO;
import com.gic.operation.web.vo.MarketAppCategoryVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.MenuApiService;
import com.gic.auth.service.UserApiService;
import com.gic.authcenter.security.core.util.UserUtils;
import com.gic.commons.util.CollectionUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.constant.EnterpriseInitCodeEnum;
import com.gic.enterprise.constant.EnterpriseListOrderByEnum;
import com.gic.enterprise.constant.EnterpriseVersionStatusEnum;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.dto.EnterpriseInitDTO;
import com.gic.enterprise.dto.EnterpriseLicenseDTO;
import com.gic.enterprise.dto.market.OrderBuyLicenseDTO;
import com.gic.enterprise.dto.market.OrderGicDTO;
import com.gic.enterprise.qo.BuyForGicQO;
import com.gic.enterprise.qo.OrderServiceQO;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.service.EnterpriseInitApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.finance.dto.TransferAccountsApprovalDTO;
import com.gic.finance.service.TransferAccountsApprovalApiService;
import com.gic.open.api.dto.ApplicationTokenDTO;
import com.gic.open.api.dto.OrderDTO;
import com.gic.open.api.dto.OrderItemDTO;
import com.gic.open.api.service.AppTokenApiService;
import com.gic.open.api.service.CouponApiService;
import com.gic.open.api.service.OrderApiService;
import com.gic.open.api.service.ServeApiService;
import com.gic.operation.web.qo.PageQO;
import com.gic.operation.web.utils.UserPasswordUtil;
import com.gic.operation.web.vo.enterprise.*;
import com.gic.store.utils.CommonResultControllerUtils;

/**
 * 商户相关操作
 * @ClassName: EnterpriseController   
 * @Description:    
 * @author guojuxing   
 * @date 2019/8/7 5:23 PM  
 */
@RestController
@RequestMapping("/enterprise")
public class EnterpriseController {
    private static final Logger LOGGER = LoggerFactory.getLogger(EnterpriseController.class);
    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private EnterpriseInitApiService enterpriseInitApiService;
    @Autowired
    private TransferAccountsApprovalApiService transferAccountsApprovalApiService;
    @Autowired
    private OrderApiService orderApiService;
    @Autowired
    private CouponApiService couponApiService;
    @Autowired
    private ServeApiService serveApiService;
    @Autowired
    private AppTokenApiService appTokenApiService;
    @Autowired
    private MenuApiService menuApiService;
    @Autowired
    private SmsSendApiService smsSendApiService;
    @Autowired
    private MarketUserApiService marketUserApiService;
    @Autowired
    private MarketAppCategoryApiService marketAppCategoryApiService;

    @RequestMapping("/save-enterprise")
    public RestResponse saveEnterprise(
            @Validated({ EnterpriseDTO.SaveEnterpriseValid.class }) EnterpriseDTO enterpriseDTO,
            @Validated({ UserDTO.UserQoValid.class }) UserDTO userDTO, String resourceJson) {
        //运维创建者ID
        enterpriseDTO.setCreatorId(UserUtils.getUser().getId());
        //前置资源解析
        if (StringUtils.isNotBlank(resourceJson)) {
            enterpriseDTO.setResourcesList(JSON.parseArray(resourceJson, EnterpriseResourceRelDTO.class));
        }
        //处理密码
        String password = UserPasswordUtil.validPassword(userDTO.getPasswordType(), userDTO.getPassword(),
                userDTO.getConfirmPassword());
        //密码加密
        userDTO.setPassword(UserPasswordUtil.getEncryptPassword(password));
        //新增商户
        ServiceResponse<Integer> result = enterpriseApiService.saveEnterprise(enterpriseDTO, userDTO);
        if (result.isSuccess() && UserConstants.CREATE_AUTO == userDTO.getPasswordType().intValue()) {
            //发送短信，通知自动生成密码
            if (StringUtils.isBlank(userDTO.getPhoneAreaCode())) {
                userDTO.setPhoneAreaCode("+86");
            }
            String[] smsArr = new String[] { enterpriseDTO.getEnterpriseName(), userDTO.getPhoneNumber(), password };
            ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms("GICpassword001",
                    result.getResult(), userDTO.getPhoneAreaCode(), userDTO.getPhoneNumber(), smsArr);
            if (!smsSendResult.isSuccess()) {
                LOGGER.warn(smsSendResult.getMessage());
            }
            return RestResponse.success(result.getResult());
        }
        return EnterpriseRestResponse.failure(result);
    }

    @RequestMapping("/list-enterprise")
    public RestResponse listEnterprise(String search, String serviceVersion, Integer serviceStatus, Integer orderBy,
            PageQO pageQO) {
        if (orderBy == null) {
            orderBy = 0;
        }
        UserDetailsVO user = UserUtils.getUser();
        //运维拥有的权限商户ID集合
        List<EnterpriseTempDTO> enterpriseTemp = user.getEnterpriseList();
        List<Integer> enterpriseIdList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(enterpriseTemp)) {
            for (EnterpriseTempDTO enterpriseTempDTO : enterpriseTemp) {
                enterpriseIdList.add(enterpriseTempDTO.getId());
            }
        }
        EnterpriseQO params = new EnterpriseQO()
                .setSearch(search).setServiceVersion(serviceVersion).setCreatorId(user.getId())
                .setEnterpriseIdList(enterpriseIdList).setOrderBy(EnterpriseListOrderByEnum.getOrderBy(orderBy))
                .setServiceStatus(serviceStatus);
        pageQO.setCurrentPage(pageQO.getCurrentPage());
        pageQO.setPageSize(pageQO.getPageSize());
        //todo 现放开过滤
        params.setListAll(1);
        return ResultControllerUtils.commonResult(enterpriseApiService.listEnterprise(params));
    }

    /**
     * 是否绑定服务市场账号
     * @return
     */
    @RequestMapping("/has-bind-market-service")
    public RestResponse hasBindMarketService(Integer enterpriseId) {
        List<Long> enterpriseIdList = new ArrayList<>();
        enterpriseIdList.add(Long.valueOf(enterpriseId));
        ServiceResponse<List<MarketUserDTO>> result = marketUserApiService.checkBindGicEnterprise(enterpriseIdList);
        if (!result.isSuccess()) {
            return EnterpriseRestResponse.failure(result);
        }
        List<MarketUserDTO> list = result.getResult();
        if (CollectionUtils.isNotEmpty(list)) {
            return RestResponse.success(list.get(0).getGicBindFlag().intValue() == 1 ? true : false);
        }
        return RestResponse.success(false);
    }

    @RequestMapping("/list-enterprise-version-status")
    public RestResponse listEnterpriseVersionStatus() {
        Map<Integer, String> resultMap = new HashMap<>(16);
        for (EnterpriseVersionStatusEnum statusEnum : EnterpriseVersionStatusEnum.values()) {
            resultMap.put(statusEnum.getCode(), statusEnum.getMessage());
        }
        return RestResponse.success(resultMap);
    }

    @RequestMapping("/list-enterprise-init-config-code")
    public RestResponse listEnterpriseInitConfigCode() {
        Map<String, String> resultMap = new HashMap<>(16);
        for (EnterpriseInitCodeEnum statusEnum : EnterpriseInitCodeEnum.values()) {
            resultMap.put(statusEnum.getCode(), statusEnum.getMessage());
        }
        return RestResponse.success(resultMap);
    }

    @RequestMapping("/init-enterprise")
    public RestResponse initEnterprise(Integer enterpriseId) {
        return ResultControllerUtils.commonResult(enterpriseApiService.initEnterprise(enterpriseId));
    }

    /**
     * 商户前置资源字典数据
     * @return
     */
    @RequestMapping("/list-enterprise-resource-dict")
    public RestResponse listEnterpriseResourceDict() {
        return ResultControllerUtils.commonResult(enterpriseApiService.getResourceDict());
    }

    @RequestMapping("/list-enterprise-init-info")
    public RestResponse listEnterpriseInitInfo(Integer enterpriseId) {
        List<EnterpriseInitDTO> dtoList = (List<EnterpriseInitDTO>) CommonResultControllerUtils
                .commonResult(enterpriseInitApiService.listByEnterpriseId(enterpriseId));
        return RestResponse.success(changeListToTree(0, dtoList));
    }

    @RequestMapping("/edit-enterprise")
    public RestResponse editEnterprise(EnterpriseDTO enterpriseDTO) {
        return ResultControllerUtils.commonResult(enterpriseApiService.editEnterprise(enterpriseDTO));
    }

    @RequestMapping("/edit-resource-switch")
    public RestResponse editResourceSwitch(Integer enterpriseId, String resources) {
        if (StringUtils.isBlank(resources)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源配置不能为空");
        }
        return ResultControllerUtils.commonResult(enterpriseApiService.editResourceSwitch(enterpriseId, resources));
    }

    @RequestMapping("/modify-enterprise-logo")
    public RestResponse modifyEnterpriseLogo(Integer enterpriseId, String logoUrl) {
        return ResultControllerUtils.commonResult(enterpriseApiService.modifyEnterpriseLogo(enterpriseId, logoUrl));
    }

    @RequestMapping("/edit-user")
    public RestResponse editUser(UserDTO userDTO) {
        //自动密码
        boolean isAutoPassword = userDTO.getPasswordType() == UserConstants.CREATE_AUTO;
        String autoPassword = null;
        if (isAutoPassword) {
            String password = UserPasswordUtil.createPasswordAuto();
            autoPassword = password;
            //密码加密
            userDTO.setPassword(UserPasswordUtil.getEncryptPassword(password));
        }
        if (userDTO.getPasswordType() != null && userDTO.getPasswordType().intValue() != UserConstants.CREATE_AUTO) {
            //如果是自定义密码
            if (StringUtils.isBlank(userDTO.getPassword())) {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "密码为空，操作有误");
            }
            userDTO.setPassword(AutoCreatePasswordUtils.encryptPassword(userDTO.getPassword()));
        }
        ServiceResponse<Void> result = userApiService.editAdmin(userDTO);
        //自动创建密码，需要发送短信
        if (result.isSuccess() && isAutoPassword) {
            Integer enterpriseId = userDTO.getEnterpriseId();
            //商户ID查询商户信息
            //商户名称
            String enterpriseName = null;
            ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
            if (enterpriseResult.isSuccess()) {
                enterpriseName = enterpriseResult.getResult().getEnterpriseName();
            } else {
                LOGGER.warn(enterpriseResult.getMessage());
            }
            String[] smsArr = new String[] { enterpriseName, userDTO.getPhoneNumber(), autoPassword };
            ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms("GICpassword001", enterpriseId,
                    userDTO.getPhoneAreaCode(), userDTO.getPhoneNumber(), smsArr);
            if (!smsSendResult.isSuccess()) {
                LOGGER.warn(smsSendResult.getMessage());
            }
        }
        return ResultControllerUtils.commonResult(result);
    }

    @RequestMapping("/get-enterprise")
    public RestResponse editEnterprise(Integer enterpriseId) {
        return RestResponse.success(
                CommonResultControllerUtils.commonResult(enterpriseApiService.getEnterpriseById(enterpriseId)));
    }

    @RequestMapping("/enable-enterprise")
    public RestResponse enableEnterprise(Integer enterpriseId) {
        return RestResponse
                .success(CommonResultControllerUtils.commonResult(enterpriseApiService.enableEnterprise(enterpriseId)));
    }

    @RequestMapping("/disable-enterprise")
    public RestResponse disableEnterprise(Integer enterpriseId) {
        return RestResponse.success(
                CommonResultControllerUtils.commonResult(enterpriseApiService.disableEnterprise(enterpriseId)));
    }

    @RequestMapping("/basic-service-info")
    public RestResponse buyForGicForNew(BuyForGicQO params, String licenseJsonArr) {
        if (StringUtils.isNotBlank(licenseJsonArr)) {
            List<OrderBuyLicenseDTO> licenseList = JSON.parseArray(licenseJsonArr, OrderBuyLicenseDTO.class);
            params.setLicenseList(licenseList);
        }
        params.setName(UserUtils.getUser().getLoginName());
        return ResultControllerUtils.commonResult(enterpriseApiService.buyForGicForNew(params));
    }

    @RequestMapping("/list-gic-default-service")
    public RestResponse listGicDefaultService() {
        return RestResponse
                .success(CommonResultControllerUtils.commonResult(serveApiService.listServePropByGICDefaultAPP()));
    }

    @RequestMapping("/enterprise-detail")
    public RestResponse enterpriseDetail(Integer enterpriseId) {
        EnterpriseDetailVO vo = new EnterpriseDetailVO();
        ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (enterpriseResult.isSuccess()) {
            //商户信息
            EnterpriseDTO enterpriseDTO = enterpriseResult.getResult();
            EnterpriseInfo enterpriseInfo = EntityUtil.changeEntityNew(EnterpriseInfo.class, enterpriseDTO);
            //商户前置资源
            ServiceResponse<List<EnterpriseResourceRelDTO>> resourceResult = enterpriseApiService
                    .getBusinessFrontRes(enterpriseId);
            if (resourceResult.isSuccess()) {
                List<EnterpriseResourceRelDTO> resourceList = resourceResult.getResult();
                if (CollectionUtils.isNotEmpty(resourceList)) {
                    enterpriseInfo.setResourceList(
                            EntityUtil.changeEntityListNew(EnterpriseResourceRelVO.class, resourceList));
                }
            }

            ServiceResponse<ApplicationTokenDTO> result = appTokenApiService
                    .getGicEnterpriseBaseApp(Long.valueOf(enterpriseId));
            if (result.isSuccess()) {
                ApplicationTokenDTO applicationTokenDTO = result.getResult();
                enterpriseInfo.setServiceName(applicationTokenDTO.getName());
            }
            vo.setEnterpriseInfo(enterpriseInfo);
            //用户信息
            ServiceResponse<UserDTO> userResult = userApiService.getUserByEnterpriseId(enterpriseId);
            if (userResult.isSuccess()) {
                UserDTO userDTO = userResult.getResult();
                UserInfo userInfo = EntityUtil.changeEntityNew(UserInfo.class, userDTO);
                vo.setUserInfo(userInfo);
            }
            //license
            ServiceResponse<List<EnterpriseLicenseDTO>> licenseResult = enterpriseApiService
                    .listEnterpriseLicense(enterpriseId);
            if (licenseResult.isSuccess()) {
                List<EnterpriseLicenseDTO> licenseList = licenseResult.getResult();
                List<LicenseInfo> licenseInfoList = EntityUtil.changeEntityListNew(LicenseInfo.class, licenseList);
                vo.setLicenseInfo(licenseInfoList);
            }
            return RestResponse.success(vo);
        } else {
            return EnterpriseRestResponse.failure(enterpriseResult);
        }
    }

    /**
     * 服务列表
     * @param enterpriseId
     * @param search 服务名称/服务商名称
     * @param appCategoryId 服务分类ID
     * @param currentPage 第几页
     * @param pageSize 每页条数
     * @return
     */
    @RequestMapping("/list-market-service")
    public RestResponse listMarketService(Integer enterpriseId, String search, Long appCategoryId, Integer currentPage,
            Integer pageSize) {
        ServiceResponse<Page<ApplicationTokenDTO>> result = appTokenApiService
                .pageGicAppServeByEid(Long.valueOf(enterpriseId), search, appCategoryId, currentPage, pageSize);
        return ResultControllerUtils.commonPageResult(result, ApplicationTokenVO.class);
    }

    /**
     * 查询服务的订购流水
     * @Title: listSerialService 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param serviceId 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/list-service-order-serial")
    public RestResponse listSerialService(Long enterpriseId, Long serviceId) {
        return ResultControllerUtils.commonResult(orderApiService.listByEidAndServeId(enterpriseId, serviceId),
                com.gic.operation.web.vo.OrderVO.class);
    }

    @RequestMapping("/list-market-app-category")
    public RestResponse listMarketAppCategory() {
        return ResultControllerUtils.commonResult(marketAppCategoryApiService.listGicMarketAppCategory(),
                MarketAppCategoryVO.class);
    }

    @RequestMapping("/enterprise-buy-info")
    public RestResponse enterpriseBuyInfo(OrderServiceQO params) {
        ServiceResponse<Page<OrderGicDTO>> result = enterpriseApiService.listOrder(params);
        if (!result.isSuccess()) {
            return EnterpriseRestResponse.failure(result);
        }
        Page<OrderGicDTO> page = result.getResult();
        Page<OrderVO> resultPage = EntityUtil.changeEntityNew(Page.class, page);

        List<OrderGicDTO> orderGicHbDTOList = page.getResult();
        List<OrderVO> orderVOList = new ArrayList<>();
        List<String> orderSerialNumberList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(orderGicHbDTOList)) {
            for (OrderGicDTO orderGicHbDTO : orderGicHbDTOList) {
                orderSerialNumberList.add(orderGicHbDTO.getOrderNum());
                //
                OrderVO orderVO = EntityUtil.changeEntityNew(OrderVO.class, orderGicHbDTO);
                orderVO.setOrderId(orderGicHbDTO.getOrderId().toString());
                orderVOList.add(orderVO);
            }
        }

        ServiceResponse<List<TransferAccountsApprovalDTO>> approvalResult = transferAccountsApprovalApiService
                .listByOrderSerialNumber(orderSerialNumberList);
        Map<String, String> approvalInfoMap = new HashMap<>(16);
        if (approvalResult.isSuccess()) {
            List<TransferAccountsApprovalDTO> approvalList = approvalResult.getResult();
            if (CollectionUtils.isNotEmpty(approvalList)) {
                for (TransferAccountsApprovalDTO dto : approvalList) {
                    approvalInfoMap.put(dto.getOrderNumber(), dto.getApprovalName());
                }
            }
        }

        if (CollectionUtils.isNotEmpty(orderVOList)) {
            for (OrderVO orderVO : orderVOList) {
                orderVO.setApprover(approvalInfoMap.get(orderVO.getOrderNum()));
            }
        }
        resultPage.setResult(orderVOList);
        return RestResponse.success(resultPage);
    }

    @RequestMapping("/list-service-version")
    public RestResponse listServiceVersion() {
        return ResultControllerUtils.commonResult(menuApiService.getGicDefaultVersion());
    }

    @RequestMapping("/list-enterprise-by-name")
    public RestResponse listEnterpriseByName(String search) {
        return ResultControllerUtils.commonResult(enterpriseApiService.listEnterpriseByName(search),
                EnterpriseInfoOfSelectVO.class);
    }

    @RequestMapping("/my-buy-info")
    public RestResponse my(OrderServiceQO params) {
        params.setOperName(UserUtils.getUser().getLoginName());
        ServiceResponse<Page<OrderGicDTO>> result = enterpriseApiService.listOrder(params);
        if (!result.isSuccess()) {
            return EnterpriseRestResponse.failure(result);
        }
        Page<OrderGicDTO> page = result.getResult();
        Page<OrderVO> resultPage = EntityUtil.changeEntityNew(Page.class, page);

        List<OrderGicDTO> orderGicHbDTOList = page.getResult();
        List<OrderVO> orderVOList = new ArrayList<>();
        List<Integer> enterpriseList = new ArrayList<>();
        List<String> orderSerialNumberList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(orderGicHbDTOList)) {
            for (OrderGicDTO orderGicHbDTO : orderGicHbDTOList) {
                enterpriseList.add(orderGicHbDTO.getEnterpriseId().intValue());
                orderSerialNumberList.add(orderGicHbDTO.getOrderNum());
                //
                OrderVO orderVO = EntityUtil.changeEntityNew(OrderVO.class, orderGicHbDTO);
                orderVO.setOrderId(orderGicHbDTO.getOrderId().toString());
                orderVOList.add(orderVO);
            }
        }
        //查询企业信息，用来获取公司名称和logo
        ServiceResponse<List<EnterpriseDTO>> enterpriseListResult = enterpriseApiService
                .listEnterpriseByIds(enterpriseList);
        List<EnterpriseDTO> enterpriseDTOList = enterpriseListResult.getResult();
        Map<Integer, EnterpriseDTO> enterpriseInfoMap = new HashMap<>(16);
        if (CollectionUtils.isNotEmpty(enterpriseDTOList)) {
            for (EnterpriseDTO enterpriseDTO : enterpriseDTOList) {
                enterpriseInfoMap.put(enterpriseDTO.getEnterpriseId(), enterpriseDTO);
            }
        }
        //查询审批人信息
        ServiceResponse<List<TransferAccountsApprovalDTO>> approvalResult = transferAccountsApprovalApiService
                .listByOrderSerialNumber(orderSerialNumberList);
        Map<String, String> approvalInfoMap = new HashMap<>(16);
        if (approvalResult.isSuccess()) {
            List<TransferAccountsApprovalDTO> approvalList = approvalResult.getResult();
            if (CollectionUtils.isNotEmpty(approvalList)) {
                for (TransferAccountsApprovalDTO dto : approvalList) {
                    approvalInfoMap.put(dto.getOrderNumber(), dto.getApprovalName());
                }
            }
        }

        if (CollectionUtils.isNotEmpty(orderVOList)) {
            for (OrderVO orderVO : orderVOList) {
                EnterpriseDTO enterpriseDTO = enterpriseInfoMap.get(orderVO.getEnterpriseId().intValue());
                orderVO.setCompanyName(enterpriseDTO.getCompanyName());
                orderVO.setLogo(enterpriseDTO.getLogo());
                orderVO.setApprover(approvalInfoMap.get(orderVO.getOrderNum()));
            }
        }
        resultPage.setResult(orderVOList);
        return RestResponse.success(resultPage);
    }

    @RequestMapping("/cancel-order")
    public RestResponse cancelOrder(Long orderId) {
        return ResultControllerUtils.commonResult(enterpriseApiService.cancelOrder(orderId));
    }

    @RequestMapping("/cancel-approval")
    public RestResponse cancelApproval(Integer id) {
        return ResultControllerUtils.commonResult(transferAccountsApprovalApiService.cancelApproval(id));
    }

    @RequestMapping("/edit-enterprise-info")
    public RestResponse editEnterpriseInfo(EnterpriseDTO enterpriseDTO) {
        return ResultControllerUtils.commonResult(enterpriseApiService.editEnterpriseInfo(enterpriseDTO));
    }

    @RequestMapping("/list-coupon")
    public RestResponse listCoupon(BigDecimal price, Long isvId, Long enterpriseId, Long serveId, Long skuId) {
        return ResultControllerUtils.commonResult(
                couponApiService.listEnterpriseCouponByServeId(price, isvId, enterpriseId, serveId, skuId),
                CouponLogVO.class);
    }

    @RequestMapping("/order-detail")
    public RestResponse orderDetail(String orderNum) {
        ServiceResponse<TransferAccountsApprovalDTO> transferResult = transferAccountsApprovalApiService
                .getByOrderNumber(orderNum);
        if (transferResult.isSuccess()) {
            TransferAccountsApprovalDTO transfer = transferResult.getResult();
            TransferApprovalServiceVO vo = EntityUtil.changeEntityNew(TransferApprovalServiceVO.class, transfer);
            //如果是空的，说明订购记录不走财务转账审批
            if (vo == null) {
                vo = new TransferApprovalServiceVO();
                vo.setOrderNumber(orderNum);
            }

            ServiceResponse<OrderDTO> orderResult = orderApiService.getOrderForFinance(orderNum);
            if (orderResult.isSuccess()) {
                OrderDTO orderDTO = orderResult.getResult();
                vo.setOrderCreateTime(orderDTO.getCreateTime());
                //支付类型
                vo.setPayType(orderDTO.getPayType());
                //支付流水号
                vo.setPayInfoSerialNumber(orderDTO.getPayNumber());

                if (vo.getOrderType() == null) {
                    //没有走财务转账审批
                    Integer serveType = orderDTO.getServeType();
                    if (serveType != null) {
                        if (serveType.intValue() == 1) {
                            vo.setOrderType(OrderTypeEnum.SERVICE.getCode());
                        } else if (serveType.intValue() == 2) {
                            vo.setOrderType(OrderTypeEnum.PACKAGE.getCode());
                        } else if (serveType.intValue() == 3) {
                            vo.setOrderType(OrderTypeEnum.EXTENSION_PACKAGE.getCode());
                        }
                    }
                }

                vo.setTotalCountFee(vo.getPlannedAmount());
                vo.setRelationOrderType(orderDTO.getOrderType());
                vo.setOperType(orderDTO.getOperType());
                vo.setOperName(orderDTO.getOperName());
                vo.setPayTime(orderDTO.getPayTime());
                vo.setPayAmount(orderDTO.getPayAmount());
                vo.setStatus(orderDTO.getStatus());
                vo.setPayDetail(orderDTO.getPayDetail());
                if (orderDTO.getOrderId() != null) {
                    vo.setOrderId(orderDTO.getOrderId().toString());
                }
                String couponName = orderDTO.getCouponName();
                if (!StringUtils.isBlank(couponName)) {
                    ServiceDiscountVO discountVO = new ServiceDiscountVO();
                    discountVO.setDiscountDetail(orderDTO.getDiscountDetail());
                    discountVO.setDiscountName(couponName);
                    vo.setServiceDiscountVO(discountVO);
                }
                if (orderDTO.getOrderItemList() != null) {
                    List<ServiceInfoVO> serviceInfoVOList = new ArrayList<>();
                    List<ExpandPackageVO> expandPackageVOList = new ArrayList<>();
                    List<LicenseVO> licenseVOList = new ArrayList<>();
                    for (OrderItemDTO itemDTO : orderDTO.getOrderItemList()) {
                        if (itemDTO.getServeType() == 1) {
                            //服务
                            ServiceInfoVO serviceInfoVO = new ServiceInfoVO();
                            serviceInfoVO.setServeName(itemDTO.getServeName());
                            serviceInfoVO.setSkuInfo(itemDTO.getSkuInfo());
                            //销售价
                            serviceInfoVO.setTotalAmount(itemDTO.getSaleAmount());
                            serviceInfoVO.setServeInfo(itemDTO.getServeInfo());
                            serviceInfoVO.setApplicationId(itemDTO.getApplicationId());
                            serviceInfoVO.setPayDetail(itemDTO.getPayDetail());
                            serviceInfoVOList.add(serviceInfoVO);
                        } else if (itemDTO.getServeType() == 2) {
                            //license
                            LicenseVO licenseVO = new LicenseVO();
                            licenseVO.setLicenseUnit(itemDTO.getLicenseUnit());
                            licenseVO.setServeName(itemDTO.getServeName());
                            //销售价
                            licenseVO.setTotalAmount(itemDTO.getSaleAmount());
                            licenseVO.setServeQuantity(itemDTO.getServeQuantity());
                            licenseVO.setServeInfo(itemDTO.getServeInfo());
                            licenseVO.setApplicationId(itemDTO.getApplicationId());
                            licenseVO.setPayDetail(itemDTO.getPayDetail());
                            licenseVOList.add(licenseVO);
                        } else if (itemDTO.getServeType() == 3) {
                            //拓展包
                            ExpandPackageVO expandPackageVO = new ExpandPackageVO();
                            expandPackageVO.setImageUrl(itemDTO.getImageUrl());
                            expandPackageVO.setServeName(itemDTO.getServeName());
                            expandPackageVO.setServeQuantity(itemDTO.getServeQuantity());
                            //销售价
                            expandPackageVO.setTotalAmount(itemDTO.getSaleAmount());
                            expandPackageVO.setServeInfo(itemDTO.getServeInfo());
                            expandPackageVO.setPayDetail(itemDTO.getPayDetail());
                            expandPackageVOList.add(expandPackageVO);
                        }
                    }
                    Map<Long, List<LicenseVO>> licenseMap;
                    if (licenseVOList.size() > 0) {
                        //license是在服务下面，进行匹配
                        licenseMap = CollectionUtil.group(licenseVOList, "applicationId");
                        vo.setLicenseVOList(licenseVOList);
                    } else {
                        licenseMap = new HashMap<>(2);
                    }
                    if (serviceInfoVOList.size() > 0) {
                        for (ServiceInfoVO serviceInfoVO : serviceInfoVOList) {
                            if (licenseMap.containsKey(serviceInfoVO.getApplicationId())) {
                                serviceInfoVO.setLicenseVOList(licenseMap.get(serviceInfoVO.getApplicationId()));
                            }
                        }
                        vo.setServiceInfoVOList(serviceInfoVOList);
                    } else if (licenseVOList.size() > 0) {
                        //没有服务，只有license数据，比如扩容
                        ServiceInfoVO temp = new ServiceInfoVO();
                        boolean isGicEnterprise = vo.getPlatformType() != null
                                && vo.getPlatformType().intValue() == EnterprisePlatformTypeEnum.GIC.getCode();
                        if (isGicEnterprise) {
                            ServiceResponse<EnterpriseDTO> tempResult = enterpriseApiService
                                    .getEnterpriseById(transfer.getEnterpriseId());
                            if (tempResult.isSuccess()) {
                                temp.setVersionName(
                                        orderDTO.getServeName() + "-" + tempResult.getResult().getVersionName());
                            } else {
                                LOGGER.warn(tempResult.getMessage());
                            }
                        } else {
                            temp.setVersionName(orderDTO.getServeName());
                        }
                        temp.setLicenseVOList(licenseVOList);
                        serviceInfoVOList.add(temp);
                        vo.setServiceInfoVOList(serviceInfoVOList);
                    }

                    if (expandPackageVOList.size() > 0) {
                        vo.setExpandPackageVOList(expandPackageVOList);
                    }
                }
                return RestResponse.success(vo);
            } else {
                return EnterpriseRestResponse.failure(orderResult);
            }
        } else {
            return EnterpriseRestResponse.failure(transferResult);
        }
    }

    public static List<Map<String, Object>> changeListToTree(int initId, List<EnterpriseInitDTO> list) {
        List<Map<String, Object>> result = new ArrayList<>();
        if (list != null) {
            for (EnterpriseInitDTO t : list) {
                if (initId == t.getParentId()) {
                    Map<String, Object> data = new HashMap<>(16);
                    data.put("parentId", t.getParentId());
                    data.put("initId", t.getEnterpriseInitId());
                    data.put("configName", t.getConfigName());
                    data.put("sort", t.getSort());
                    data.put("isInitConfig", t.getIsInitConfig());
                    data.put("level", t.getLevel());
                    data.put("configUrl", t.getConfigUrl());
                    data.put("configCode", t.getConfigCode());
                    List<Map<String, Object>> children = changeListToTree(t.getEnterpriseInitId(), list);
                    data.put("children", children);
                    result.add(data);
                }
            }
        }
        return result;
    }
}
