package com.gic.operation.web.controller;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.open.api.service.market.MarketUserApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.MenuApiService;
import com.gic.auth.service.OperationUserApiService;
import com.gic.auth.service.UserApiService;
import com.gic.authcenter.security.core.dto.UserDetailsVO;
import com.gic.authcenter.security.core.util.UserUtils;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.base.EnterpriseInfo;
import com.gic.enterprise.base.MenuInfo;
import com.gic.enterprise.base.UserInfo;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;

@RestController
public class LoginController {
    @Autowired
    private OperationUserApiService operationUserApiService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private MenuApiService menuApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private MarketUserApiService marketUserApiService;

    private static final String SSO_LOGIN_URL = "https://four.gicdev.com/gic-auth-web/login-for-operation";

    /**
     * 商户绑定服务市场
     * @Title: bind 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param nationCode
     * @param phone
     * @param password 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("bind-enterprise-to-service-market")
    public RestResponse bind(Integer enterpriseId, String nationCode, String phone, String password)
            throws IOException {
        return ResultControllerUtils.commonResult(
                marketUserApiService.BindGicEnterprise(Long.valueOf(enterpriseId), nationCode.replace("+", ""), phone, password));
    }

    /**
     * 单点登录到服务市场首页
     * @Title: ssoToMarketService 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("sso-to-market-service")
    public RestResponse ssoToMarketService(Integer enterpriseId) throws IOException {
        ServiceResponse<String> result = marketUserApiService.BindGicEnterprise(Long.valueOf(enterpriseId), null, null,
                null);
        if (result.isSuccess()) {
            return RestResponse.success("https://four.gicdev.com/market/gic" + "?loginKey=" + result.getResult()
                    + "&enterpriseId=" + enterpriseId);
        }
        return EnterpriseRestResponse.failure(result);
    }

    @RequestMapping("login-gic")
    public RestResponse login(Integer enterpriseId, String redirectUrl) throws IOException {
        if (StringUtils.isBlank(redirectUrl)) {
            redirectUrl = "https://four.gicdev.com/damo-system/overview/index";
//            redirectUrl = "https://four.gicdev.com/cost-center/billing-center/overview";
        }
        String token;

        UserDetailsVO userDetailsVO = UserUtils.getUser();
        String phone = userDetailsVO.getUserMobile();
        //运营人员国内，默认数据
        ServiceResponse<UserDTO> userResult = userApiService.listUserByPhoneNumber(Constants.NATION_CODE, phone, enterpriseId);
        if (userResult.isSuccess()) {
            //获取运营实施信息
            UserDTO user = userResult.getResult();
            if (user == null) {
                //新建运营实施账号
                UserDTO temp = new UserDTO();
                temp.setEnterpriseId(enterpriseId);
                temp.setPhoneNumber(phone);
                temp.setUserName(userDetailsVO.getRealName());
                temp.setPhoneAreaCode(Constants.NATION_CODE);
                temp.setSuperAdmin(1);
                ServiceResponse<Integer> saveResult = userApiService.saveOperationUser(temp);
                user = temp;
                user.setUserId(saveResult.getResult());
            }

            //gic用户信息塞入
            UserDetail userDetail = new UserDetail();
            //设置登录用户信息
            userDetail.setUserId(user.getUserId());
            userDetail.setUserInfo(EntityUtil.changeEntityNew(UserInfo.class, user));
            //设置企业资料
            ServiceResponse<EnterpriseDTO> enterprise = this.enterpriseApiService.getEnterpriseById(enterpriseId);
            if (enterprise.isSuccess()) {
                EnterpriseDTO enterpriseDTO = enterprise.getResult();
                if (enterpriseDTO != null) {
                    userDetail.setEnterpriseInfo(EntityUtil.changeEntityNew(EnterpriseInfo.class, enterpriseDTO));
                }
                //设置权限
                ServiceResponse<List<MenuDTO>> menuResult = menuApiService.listUserMenu(user.getUserId(),
                        enterpriseDTO.getVersionCode());
                if (menuResult.isSuccess()) {
                    List<MenuDTO> menuList = menuResult.getResult();
                    List<MenuInfo> menuInfoList = EntityUtil.changeEntityListNew(MenuInfo.class, menuList);
                    Map<String, Object> map = new HashMap<>(16);
                    if (CollectionUtils.isNotEmpty(menuInfoList)) {
                        for (MenuInfo menuDTO : menuInfoList) {
                            if (StringUtils.isBlank(menuDTO.getMenuUrl())) {
                                continue;
                            }
                            map.put(menuDTO.getMenuUrl(), menuDTO);
                        }
                    }
                    //查询全部的操作模块
                    Map<String, MenuInfo> moduleMap = new HashMap<>(16);
                    ServiceResponse<List<MenuDTO>> allGicMenu = menuApiService.getSuperAdminMenuNotTree(enterpriseId);
                    if (allGicMenu.isSuccess()) {
                        List<MenuDTO> temp = allGicMenu.getResult();
                        List<MenuInfo> tempMenuInfoList = EntityUtil.changeEntityListNew(MenuInfo.class, temp);
                        if (CollectionUtils.isNotEmpty(tempMenuInfoList)) {
                            for (MenuInfo menuDTO : tempMenuInfoList) {
                                if (StringUtils.isBlank(menuDTO.getMenuUrl())) {
                                    continue;
                                }
                                moduleMap.put(menuDTO.getProjectUrlForWeb() + menuDTO.getMenuUrl(), menuDTO);
                            }
                        }
                        userDetail.setMenuInfoList(tempMenuInfoList);
                    }
                    //塞值
                    userDetail.setMenuUrlMap(map);
                    userDetail.setModuleUrlMap(moduleMap);
                }
            }
            //塞缓存
            token = UserDetailUtils.setUserDetail(userDetail);
            return RestResponse.success(SSO_LOGIN_URL + "?token=" + token + "&redirectUrl=" + redirectUrl);
        }
        return RestResponse.success();
    }
}
